package com.emato.cuspay.biz.wx;

import com.emato.cuspay.biz.CuspayBiz;
import com.emato.cuspay.entity.merch.MerchPayCfg;
import com.emato.cuspay.service.MerchPaymentService;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;

/**
 * 加载缓存
 * @author zx
 * @version 1.0
 * 2018-05-23 10:00
 */
public abstract class AbstractCusDeclareBiz implements CuspayBiz {

    @Autowired
    private MerchPaymentService paymentService;

    //商户支付信息缓存
    private Map<String, MerchPayCfg> payCaches = Maps.newHashMap();

    //重新加载缓存
    @PostConstruct
    private void reloadPayCaches() {
        if (this.paymentService == null)
            throw new NullPointerException("paymentService is null");
        List<MerchPayCfg> list = this.paymentService.loadMerchPayCfg();
        if (list != null && !list.isEmpty()) {
            list.forEach(cfg ->{
                payCaches.put(cfg.getMerchSn(), cfg);
            });
        }
    }

    //获取商户支付数据
    protected MerchPayCfg getMerchPayCfgCache(String sn) {
        MerchPayCfg merchPayCfg = payCaches.get(sn);
        if(merchPayCfg == null) {
            this.reloadPayCaches();
            return payCaches.get(sn);
        }
        return merchPayCfg;
    }


    public MerchPaymentService getPaymentService() {
        return paymentService;
    }

    public void setPaymentService(MerchPaymentService paymentService) {
        this.paymentService = paymentService;
    }

}

