package com.emato.cuspay.biz.wx;

import com.emato.cuspay.biz.CuspayBiz;
import com.emato.cuspay.common.contant.MerchNoticeDict;
import com.emato.cuspay.common.contant.TablePrimaryKeyPrefix;
import com.emato.cuspay.common.core.db.IdWorkerAide;
import com.emato.cuspay.entity.merch.MerchNoti;
import com.emato.cuspay.entity.merch.MerchPayCfg;
import com.emato.cuspay.service.MerchNotiService;
import com.emato.cuspay.util.OkHttpUtils;
import com.emato.cuspay.util.ReaderXmlForDOM4J;
import com.emato.cuspay.util.XmlUtils;
import com.emato.cuspay.wx.common.MerchNotiBuilder;
import com.emato.cuspay.wx.common.WxContants;
import com.emato.cuspay.wx.common.WxDict;
import com.emato.cuspay.wx.dto.WxQueryResponseMsg;
import com.emato.cuspay.wx.dto.WxQuerySuccessResponseMsgDto;
import com.emato.cuspay.wx.dto.WxResponseMsg;
import com.emato.cuspay.wx.entity.WxCbPayDoc;
import com.emato.cuspay.wx.entity.WxPayError;
import com.emato.cuspay.wx.service.WxCbPayDocService;
import com.emato.cuspay.wx.service.WxPayErrorService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.xml.bind.JAXB;
import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;

/**
 * 微信订单信息查询
 * @author hyq
 * @version 1.0
 * 2018-05-23 17:46
 */
@Component
public class WxCusDeclareQueryBiz extends AbstractCusDeclareBiz implements CuspayBiz {
    private static final Logger logger = LoggerFactory.getLogger(WxCusDeclareQueryBiz.class);

    @Value("${db.wx.declare.limit}")
    private Integer limit;

    @Value("${db.wx.declare.count}")
    private Integer count;

    @Value("${wx.payment.query.url}")
    private String queryURL;

    @Autowired
    private WxCbPayDocService wxCbPayDocService;

    @Autowired
    private WxPayErrorService wxPayErrorService;

    @Autowired
    private MerchNotiService merchNotiService;

    /**
     * 微信订单信息查询
     */
    public void biz() {
        limit = (limit != null && limit > 0) ? limit : 20;
        count = (count != null && count > 0) ? count : 10;

        //查出等待处理、业务处理中的支付单证信息
        List<WxCbPayDoc> wxCbPayDocs = null;
        try {
            wxCbPayDocs = wxCbPayDocService.selectPayDocByDocStatus(limit);
            if (wxCbPayDocs == null || wxCbPayDocs.isEmpty()) {
                logger.info("未查到等待处理、业务处理中微信推海关的支付数据");
                return;
            }
        } catch (Exception e) {
            logger.error("查询等待处理、业务处理中微信推海关的支付数据异常", e);
            return;
        }

        //请求微信查询接口

        //记录异常数据
        List<WxPayError> errorList = Lists.newArrayList();

        wxCbPayDocs.forEach(wxCbPayDoc -> {
            WxQuerySuccessResponseMsgDto querySuccessResponseDto = new WxQuerySuccessResponseMsgDto();
            //获取商户信息 从缓存中去  缓存不存在 从数据库重取
            MerchPayCfg merchPayCfg = getMerchPayCfgCache(wxCbPayDoc.getMerchSn());

            MerchNotiBuilder builder = new MerchNotiBuilder();

            //设置支付信息
            builder.wxCbPay(wxCbPayDoc);
            //设置通知次数
            builder.notiCount(count);

            if (merchPayCfg == null) {
                wxCbPayDoc.setReturnCode("FAIL");
                wxCbPayDoc.setReturnMsg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在");
                wxCbPayDocService.updateWxCbPay(wxCbPayDoc);
                logger.error("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付配置信息不存在 ");
                return;
            }

            if (merchPayCfg.getMerchWxApiKey() == null) {
                wxCbPayDoc.setReturnCode(WxContants.ERROR_NO_INFOMATION);
                wxCbPayDoc.setReturnMsg("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付信息api密钥为空");
                wxCbPayDocService.updateWxCbPay(wxCbPayDoc);
                logger.error("商户编号为【"+wxCbPayDoc.getMerchSn()+"】的商户支付信息api密钥为空 ");
                return;
            }
            //设置商户配置信息
            builder.merchPay(merchPayCfg);

            //组装xml 格式数据
            String xml = XmlUtils.map2Xml(assemblyDeclareQueryXml(wxCbPayDoc, merchPayCfg.getMerchWxApiKey()));
            logger.info("微信查询订单请求数xml据为:"+xml);

            //通过http post请求 发送xml数据
            Request request = OkHttpUtils.buildRequest(queryURL,
                    RequestBody.create(MediaType.parse("application/xml; charset=utf-8"), xml));

            WxQueryResponseMsg wxQueryResponseMsg = null;
            String result = "";
            try {
                result = OkHttpUtils.post(request);
                StringReader reader = new StringReader(result);
                wxQueryResponseMsg = JAXB.unmarshal(reader, WxQueryResponseMsg.class);
                logger.info("请求微信查询订单接口返回result:"+result + "；xml:"+wxQueryResponseMsg);
                if (wxQueryResponseMsg == null) {
                    throw new IOException("http请求["+queryURL+"]响应结果为空" );
                }
            } catch (IOException e) {
                logger.error("数据请求异常，查询订单请求数据为xml{}", xml, e);
                WxPayError wxPayError = createWxPayError(wxCbPayDoc, querySuccessResponseDto);
                errorList.add(wxPayError);
            }
            MerchNoti merchNoti = new MerchNoti();
            try {
                //返回状态码为成功
                if(WxContants.SUCCESS.equals(wxQueryResponseMsg.getReturnCode())){

                    wxCbPayDoc.setResultCode(wxQueryResponseMsg.getResultCode());
                    wxCbPayDoc.setReturnCode(wxQueryResponseMsg.getReturnCode());
                    wxCbPayDoc.setReturnMsg(wxQueryResponseMsg.getReturnMsg());
                    wxCbPayDoc.setErrCode(wxQueryResponseMsg.getErrCode());
                    wxCbPayDoc.setErrCodeDes(wxQueryResponseMsg.getErrCodeDes());
                    //业务状态码为成功
                    if(WxContants.SUCCESS.equals(wxQueryResponseMsg.getResultCode())){
                        wxCbPayDoc.setTransactionId(wxQueryResponseMsg.getTransactionId());

                        //订购人与支付人校验结果
                        if (wxCbPayDoc.getBuyerPayerCheck() != null) {
                            if (MerchNoticeDict.BuyerPayerCheckStatus.i_0.getItem().equals(wxCbPayDoc.getBuyerPayerCheck())) { //未知
                                builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_0.getItem());
                            }
                            if (MerchNoticeDict.BuyerPayerCheckStatus.i_1.getItem().equals(wxCbPayDoc.getBuyerPayerCheck())) {//一致
                                builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_1.getItem());
                            }
                            if (MerchNoticeDict.BuyerPayerCheckStatus.i_2.getItem().equals(wxCbPayDoc.getBuyerPayerCheck())) {//不一致
                                builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_2.getItem());
                            }
                            if(MerchNoticeDict.BuyerPayerCheckStatus.i_3.getItem().equals(wxCbPayDoc.getBuyerPayerCheck())){//校验异常
                                builder.certCheckResult(MerchNoticeDict.BuyerPayerCheckStatus.i_3.getItem());
                                wxCbPayDoc.setBuyerPayerCheck(MerchNoticeDict.BuyerPayerCheckStatus.i_3.getItem());
                            }
                        }

                        int msgCount = wxQueryResponseMsg.getCount();
                        for (int i = 0; i < msgCount; i++){
                            //解析xml获取WxQuerySuccessResponseMsgDto
                            querySuccessResponseDto = ReaderXmlForDOM4J.parse(result, i);
                            wxCbPayDoc.setSubOrderNo(querySuccessResponseDto.getSubOrderNo());
                            wxCbPayDoc.setMchCustomsNo(querySuccessResponseDto.getMchCustomsNo());
                            wxCbPayDoc.setCustoms(querySuccessResponseDto.getCustoms());
                            wxCbPayDoc.setFeeType(querySuccessResponseDto.getFeeType());
                            wxCbPayDoc.setOrderFee(querySuccessResponseDto.getOrderFee());
                            wxCbPayDoc.setDuty(querySuccessResponseDto.getDuty());
                            wxCbPayDoc.setTransportFee(querySuccessResponseDto.getTransportFee());
                            wxCbPayDoc.setProductFee(querySuccessResponseDto.getProductFee());

                            String state = querySuccessResponseDto.getState();

                            if (WxDict.ResponseMsgState.UNDECLARED.getItem().equals(state)) {//返回状态未申报
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_10.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(WxDict.ResponseMsgState.UNDECLARED.getItemName())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_10.getItem())
                                        .build();
                            }else if (WxDict.ResponseMsgState.PROCESSING.getItem().equals(state)) {// 申报中
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_11.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(WxDict.ResponseMsgState.PROCESSING.getItemName())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_11.getItem())
                                        .build();
                            }else if(WxDict.ResponseMsgState.SUBMITTED.getItem().equals(state)) {//申报已提交
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_10.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(WxDict.ResponseMsgState.SUBMITTED.getItemName())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_11.getItem())
                                        .build();

                            }else if (WxDict.ResponseMsgState.SUCCESS.getItem().equals(state)) {//申报成功
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_12.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(wxQueryResponseMsg.getErrCodeDes())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_12.getItem())
                                        .build();
                            }else if (WxDict.ResponseMsgState.FAIL.getItem().equals(state)) {//申报失败
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(WxDict.ResponseMsgState.FAIL.getItemName())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_13.getItem())
                                        .build();
                            }else if (WxDict.ResponseMsgState.EXCEPT.getItem().equals(state)) {//海关接口异常
                                wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                                merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                        .msg(WxDict.ResponseMsgState.EXCEPT.getItemName())
                                        .cusDeclStatus(WxDict.MerchNoticeStatus.i_13.getItem())
                                        .build();
                            }
                        }
                    }else{//业务状态码失败
                        wxCbPayDoc.setErrCode(wxQueryResponseMsg.getErrCode());
                        wxCbPayDoc.setErrCodeDes(wxQueryResponseMsg.getErrCodeDes());
                        wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_13.getItem());

                        merchNoti = builder.code(wxQueryResponseMsg.getErrCode())
                                .msg(wxQueryResponseMsg.getErrCodeDes())
                                .cusDeclStatus(WxDict.MerchNoticeStatus.i_13.getItem())
                                .build();
                    }
                }
                ///返回状态码为失败
                if (WxContants.RETURN_CODE_FAIL.equals(wxQueryResponseMsg.getReturnCode()) ||
                        WxContants.FAIL.equals(wxQueryResponseMsg.getReturnCode())) {
                    wxCbPayDoc.setDocStatus(WxDict.PaymentDocStatus.i_03.getItem());
                    //通知商户
                    merchNoti = builder.code(WxContants.ERROR_FAIL)
                            .msg(wxQueryResponseMsg.getReturnMsg())
                            .cusDeclStatus(WxDict.MerchNoticeStatus.i_13.getItem())
                            .build();
                }

                wxCbPayDocService.updateWxCbPay(wxCbPayDoc);
            } catch (Exception e) {
                logger.error("更新支付单证入库信息状态异常", e);
                return;
            }

            List<MerchNoti> mchNotiList = null;
            try {
                MerchNoti noti = new MerchNoti();
                noti.setCode(wxQueryResponseMsg.getErrCode());
                noti.setAllPayNo(wxQueryResponseMsg.getTransactionId());
                noti.setCusDeclStatus(merchNoti.getCusDeclStatus());
                mchNotiList = merchNotiService.getMerchNotiByCodeAndOrderNo(noti);
            } catch (Exception e) {
                logger.error("根据code查询商户通知信息异常", e);
                return;
            }

            try {
                if(mchNotiList != null && mchNotiList.size() > 0){
                    merchNoti.setNotiSn(mchNotiList.get(0).getNotiSn());
                    merchNotiService.update(merchNoti);
                }else{
                    merchNotiService.insert(merchNoti);
                }
            } catch (Exception e) {
                logger.error("操作商户通知数据异常", e);
                WxPayError wxPayError = createWxPayError(wxCbPayDoc, querySuccessResponseDto);
                wxPayError.setErrMsg("操作商户通知数据异常");
                errorList.add(wxPayError);
            }
        });
        //存储异常记录
        try {
            if (errorList != null && !errorList.isEmpty()) {
                int errResult = wxPayErrorService.insertWxPayErrorBatch(errorList);
                if (errResult == 0) {
                    logger.error("持久化支付申报异常数据失败data:" + errorList);
                    throw new Exception("持久化支付申报异常数据失败data:" + errorList);
                }
            }
        } catch (Exception e) {
            logger.error("持久化支付申报异常数据失败data:" + errorList, e);
            throw new RuntimeException("持久化支付申报异常数据失败data:",e);
        }

    }
    /**
     * 记录异常的支付单证信息
     * @param wxCbPayDoc
     * @return
     */
    private WxPayError createWxPayError(WxCbPayDoc wxCbPayDoc, WxQuerySuccessResponseMsgDto querySuccessDto) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        WxPayError wxPayError = new WxPayError();
        wxPayError.setErrorSn(TablePrimaryKeyPrefix.wx_pay_error_type + IdWorkerAide.nextId());
        wxPayError.setWxPaySn(wxCbPayDoc.getTransactionId());
        wxPayError.setMerchSn(wxCbPayDoc.getMerchSn());
        wxPayError.setMerchName(wxCbPayDoc.getMerchName());
        wxPayError.setAppid(wxCbPayDoc.getAppid());
        wxPayError.setMchId(wxCbPayDoc.getMchId());
        wxPayError.setOutTradeNo(wxCbPayDoc.getOutTradeNo());
        wxPayError.setTransactionId(wxCbPayDoc.getTransactionId());
        wxPayError.setSubOrderNo(querySuccessDto.getSubOrderNo());
        wxPayError.setCustoms(querySuccessDto.getCustoms());
        wxPayError.setErrCode(wxCbPayDoc.getErrCode());
        wxPayError.setErrMsg(wxCbPayDoc.getReturnMsg());
        wxPayError.getCreateTime(sdf.format(new Date()));
        return  wxPayError;
    }

    /**
     * 组装微信查询订单所需要的参数数据
     * @param wxCbPayDoc 微信支付数据
     * @param key 微信Api密钥
     * @return
     */
    protected Map<String, String> assemblyDeclareQueryXml(WxCbPayDoc wxCbPayDoc, String key) {
        //1.选择报关数据,排序
        SortedMap<String, String> sorted = Maps.newTreeMap();
        sorted.put("sign_type", "MD5");
        sorted.put("appid", wxCbPayDoc.getAppid());
        sorted.put("mch_id", wxCbPayDoc.getMchId());
        sorted.put("out_trade_no", wxCbPayDoc.getOutTradeNo());
        sorted.put("transaction_id", wxCbPayDoc.getTransactionId());
        sorted.put("customs", wxCbPayDoc.getCustoms());

        //2.生成签名
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<String,String> entry : sorted.entrySet()) {
            sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
        }
        sb.append("key").append("=").append(key);
        //签名MD5 加密
        String sign = DigestUtils.md5Hex(sb.toString()).toUpperCase();

        sorted.put("sign", sign);
        return sorted;
    }


}
